# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.version
:Synopsis:          This simple script contains the package version
:Usage:             ``from .utils import version``
:Example:           ``__version__ = version.get_full_version()``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     24 Nov 2022
"""

import json
import urllib.request

from . import log_utils

# Define special and global variables
__version__ = "5.2.0rc1"
latest_version_reported = False
logger = log_utils.initialize_logging(__name__)


def get_full_version():
    """This function returns the current full version of the khoros package."""
    return __version__


def log_current_version(debug=False):
    """This function reports the current running version of the library in a debug log entry.

    .. versionchanged:: 5.0.0
       Removed the redundant ``return`` statement.

    .. versionadded:: 3.0.0

    :param debug: Defines if the message should be logged with the ``DEBUG`` log level. (``False`` by default)
    :type debug: bool
    :returns: None
    """
    log_msg = f'The current version of the library is {__version__}.'
    if debug:
        logger.debug(log_msg)
    else:
        logger.info(log_msg)


def get_major_minor_version():
    """This function returns the current major.minor (i.e. X.Y) version of the khoros package."""
    return ".".join(__version__.split(".")[:2])


def get_latest_stable():
    """This function returns the latest stable version of the khoros package.

    .. versionchanged:: 3.4.0
       This function has been refactored to leverage the standard library instead of the ``requests`` library.

    .. versionchanged:: 3.0.0
       Error handling and logging was added to avoid an exception if PyPI cannot be queried successfully.

    :returns: The latest stable version in string format
    """
    try:
        response = urllib.request.urlopen('https://pypi.org/pypi/khoros/json')
        pypi_data = json.loads(response.read().decode(response.info().get_param('charset') or 'utf-8'))
        latest_stable = pypi_data['info']['version']
        global latest_version_reported
        if not latest_version_reported:
            logger.debug(f'The latest stable version of the library on PyPI is {latest_stable}.')
            latest_version_reported = True
    except Exception as exc:
        exc_msg = f"{type(exc).__name__} - {exc}"
        logger.error("Unable to perform the query to retrieve the latest stable version of the library due "
                     f"to the following exception: {exc_msg}")
        latest_stable = '0.0.0'
    return latest_stable


def latest_version():
    """This function defines if the current version matches the latest stable version on PyPI.

    .. versionchanged:: 3.0.0
       The function was reduced to a single return statement.

    :returns: Boolean value indicating if the versions match
    """
    return True if get_full_version() == get_latest_stable() else False


def warn_when_not_latest():
    """This function displays a :py:exc:`RuntimeWarning` if the running version doesn't match the latest stable version.

    .. versionchanged:: 5.0.0
       Removed the redundant ``return`` statement and merged two ``if`` statements.

    .. versionchanged:: 3.0.0
       The function was updated to use logging for the warning rather than the :py:mod:`warnings` module.

    :returns: None
    """
    if not latest_version() and get_latest_stable() != '0.0.0':
        warn_msg = "The latest stable version of khoros is not running. " + \
                   "Consider running 'pip install khoros --upgrade' when feasible."
        logger.warning(warn_msg)


# Log the current version only when debug mode is enabled
log_current_version(debug=True)



# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.tests.test_users
:Synopsis:          This module is used by pytest to verify that the ``users`` module functions properly
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     03 Oct 2022
"""

import os
import sys

import pytest
import requests

from . import resources

# Define a global variable to define when the package path has been set
package_path_defined = False

# Define constants
USER_ID = 216
USERNAME = 'joeCustomer'


def set_package_path():
    """This function adds the high-level khoros directory to the sys.path list.

    .. versionadded:: 5.1.2
    """
    global package_path_defined
    if not package_path_defined:
        sys.path.insert(0, os.path.abspath('../..'))
        package_path_defined = True


def test_impersonate_user():
    """This function tests the ability to impersonate a user.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Perform the API call to impersonate a user
    khoros_object.users.impersonate_user('joeCustomer')


def test_create_user(monkeypatch):
    """This function tests the ability to create a user.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Overwrite the requests.get functionality with the mock_post() function
    monkeypatch.setattr(requests, 'post', resources.mock_success_post)

    # Perform the API call and assert that it was successful
    response = khoros_object.users.create(
        login='testUser',
        email='test@test.com',
        first_name='Test',
        last_name='User')
    assert response.get('status') == 'success'


def test_failed_create_user(monkeypatch):
    """This function verifies that the appropriate exception is raised if the user creation process fails.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Overwrite the requests.get functionality with the mock_post() function
    monkeypatch.setattr(requests, 'post', resources.mock_error_post)

    # Perform the API call and assert that it was successful
    with pytest.raises(exceptions.UserCreationError):
        khoros_object.users.create(
            login='testUser',
            email='test@test.com',
            first_name='Test',
            last_name='User')


def test_unsupported_update_sso_id():
    """This function tests to ensure the ``CurrentlyUnsupportedError`` exception is raised when trying to update the
    SSO ID of a user.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Perform the API call and assert that the exception is raised
    with pytest.raises(exceptions.CurrentlyUnsupportedError):
        khoros_object.users.update_sso_id('abcdEFGH', user_login=USERNAME)


def test_get_user_identifiers():
    """This function tests the ability to retrieve the identifiers for a user.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Retrieve the User ID and assert the response was expected
    user_id = khoros_object.users.get_user_id(login=USERNAME)
    assert isinstance(user_id, int) and user_id > 0
    # TODO: Troubleshoot why the test below fails with a LiQL invalid query syntax error
    # user_id = khoros_object.users.get_user_id(first_name='Joe', last_name='Customer')
    # assert isinstance(user_id, int) and user_id > 0

    # Retrieve the email address through various methods and assert the response was expected
    email = khoros_object.users.get_email(user_id=user_id)
    assert isinstance(email, str) and '@' in email
    email = khoros_object.users.get_email(login=USERNAME)
    assert isinstance(email, str) and '@' in email

    # Retrieve the User ID using email and assert the response was expected
    user_id = khoros_object.users.get_user_id(email=email)
    assert isinstance(user_id, int) and user_id > 0

    # Retrieve the username through various methods and assert the responses
    username = khoros_object.users.get_username(user_id=user_id)
    assert username == 'joeCustomer'
    username = khoros_object.users.get_login(user_id=user_id)
    assert username == 'joeCustomer'
    username = khoros_object.users.get_username(email=email)
    assert username == 'joeCustomer'


def test_users_table_query(monkeypatch):
    """This function tests the ability to query the users table.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Overwrite the requests.get functionality with the mock_post() function
    monkeypatch.setattr(requests, 'get', resources.mock_success_post)

    response = khoros_object.users.query_users_table_by_id('login', USER_ID)
    assert response.get('status') == 'success'
    response = khoros_object.users.query_users_table_by_id(['login', 'email'], USER_ID)
    assert response.get('status') == 'success'


def test_get_counts():
    """This function tests the various functions that involve retrieving user-related counts.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Test retrieving the album count and verifying the response
    album_count = khoros_object.users.get_album_count(user_id=USER_ID)
    assert isinstance(album_count, int) and album_count >= 0

    # Test retrieving the follower count and verifying the response
    followers_count = khoros_object.users.get_followers_count(user_id=USER_ID)
    assert isinstance(followers_count, int) and followers_count >= 0

    # Test retrieving the following count and verifying the response
    following_count = khoros_object.users.get_following_count(user_id=USER_ID)
    assert isinstance(following_count, int) and following_count >= 0

    # Test retrieving the images count and verifying the response
    image_count = khoros_object.users.get_images_count(user_id=USER_ID)
    assert isinstance(image_count, int) and image_count >= 0
    image_count = khoros_object.users.get_public_images_count(user_id=USER_ID)
    assert isinstance(image_count, int) and image_count >= 0

    # Test retrieving the messages count and verifying the response
    msg_count = khoros_object.users.get_messages_count(user_id=USER_ID)
    assert isinstance(msg_count, int) and msg_count >= 0

    # Test retrieving the roles count and verifying the response
    roles_count = khoros_object.users.get_roles_count(user_id=USER_ID)
    assert isinstance(roles_count, int) and roles_count >= 0

    # Test retrieving the authored solutions count and verifying the response
    solutions_count = khoros_object.users.get_solutions_authored_count(user_id=USER_ID)
    assert isinstance(solutions_count, int) and solutions_count >= 0

    # Test retrieving the posted topics count and verifying the response
    topics_count = khoros_object.users.get_topics_count(user_id=USER_ID)
    assert isinstance(topics_count, int) and topics_count >= 0

    # Test retrieving the posted replies count and verifying the response
    # TODO: Troubleshoot why the response below returns TypeError: list indices must be integers or slices, not str
    # replies_count = khoros_object.users.get_replies_count(user_id=USER_ID)
    # assert isinstance(replies_count, int) and replies_count >= 0

    # Test retrieving the posted videos count and verifying the response
    videos_count = khoros_object.users.get_videos_count(user_id=USER_ID)
    assert isinstance(videos_count, int) and videos_count >= 0

    # Test retrieving the kudos given count and verifying the response
    kudos_given_count = khoros_object.users.get_kudos_given_count(user_id=USER_ID)
    assert isinstance(kudos_given_count, int) and kudos_given_count >= 0

    # Test retrieving the kudos given count and verifying the response
    kudos_received_count = khoros_object.users.get_kudos_received_count(user_id=USER_ID)
    assert isinstance(kudos_received_count, int) and kudos_received_count >= 0

    # Test retrieving the online users count
    online_users_count = khoros_object.users.get_online_user_count()
    assert isinstance(online_users_count, int)


# Import the exceptions modules
exceptions = resources.import_modules('khoros.errors.exceptions')



# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.tests.test_exceptions
:Synopsis:          This module is used by pytest to verify that the exceptions can be raised properly
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     03 Oct 2022
"""

import pytest

from ...errors import exceptions


def test_raising_exceptions():
    """This function tests that exceptions are raised successfully.

    .. versionadded:: 5.1.2
    """
    # Test raising the InvalidStructureTypeError exception
    with pytest.raises(exceptions.InvalidStructureTypeError):
        raise exceptions.InvalidStructureTypeError()
    with pytest.raises(exceptions.InvalidStructureTypeError):
        raise exceptions.InvalidStructureTypeError(val='testing')

    # Test raising the InvalidCallbackURLError exception
    with pytest.raises(exceptions.InvalidCallbackURLError):
        raise exceptions.InvalidCallbackURLError()
    with pytest.raises(exceptions.InvalidCallbackURLError):
        raise exceptions.InvalidCallbackURLError(val='testing')

    # Test raising the MissingAuthDataError exception
    with pytest.raises(exceptions.MissingAuthDataError):
        raise exceptions.MissingAuthDataError()

    # Test raising the SessionAuthenticationError exception
    with pytest.raises(exceptions.SessionAuthenticationError):
        raise exceptions.SessionAuthenticationError()
    with pytest.raises(exceptions.SessionAuthenticationError):
        raise exceptions.SessionAuthenticationError(message='testing')

    # Test raising the SsoAuthenticationError exception
    with pytest.raises(exceptions.SsoAuthenticationError):
        raise exceptions.SsoAuthenticationError()
    with pytest.raises(exceptions.SsoAuthenticationError):
        raise exceptions.SsoAuthenticationError(message='testing')

    # Test raising the CurrentlyUnsupportedError exception
    with pytest.raises(exceptions.CurrentlyUnsupportedError):
        raise exceptions.CurrentlyUnsupportedError()
    with pytest.raises(exceptions.CurrentlyUnsupportedError):
        raise exceptions.CurrentlyUnsupportedError('testing')
    with pytest.raises(exceptions.CurrentlyUnsupportedError):
        raise exceptions.CurrentlyUnsupportedError(message='testing')

    # Test raising the DataMismatchError exception
    with pytest.raises(exceptions.DataMismatchError):
        raise exceptions.DataMismatchError()
    with pytest.raises(exceptions.DataMismatchError):
        raise exceptions.DataMismatchError(data='testing')
    with pytest.raises(exceptions.DataMismatchError):
        raise exceptions.DataMismatchError(data=['testing1', 'testing2'])

    # Test raising the InvalidParameterError exception
    with pytest.raises(exceptions.InvalidParameterError):
        raise exceptions.InvalidParameterError()
    with pytest.raises(exceptions.InvalidParameterError):
        raise exceptions.InvalidParameterError(val='testing')

    # Test raising the InvalidFieldError exception
    with pytest.raises(exceptions.InvalidFieldError):
        raise exceptions.InvalidFieldError()
    with pytest.raises(exceptions.InvalidFieldError):
        raise exceptions.InvalidFieldError(val='testing')

    # Test raising the InvalidURLError exception
    with pytest.raises(exceptions.InvalidURLError):
        raise exceptions.InvalidURLError()
    with pytest.raises(exceptions.InvalidURLError):
        raise exceptions.InvalidURLError(url='khoros.com')

    # Test raising the MissingRequiredDataError exception
    with pytest.raises(exceptions.MissingRequiredDataError):
        raise exceptions.MissingRequiredDataError()
    with pytest.raises(exceptions.MissingRequiredDataError):
        raise exceptions.MissingRequiredDataError('init')
    with pytest.raises(exceptions.MissingRequiredDataError):
        raise exceptions.MissingRequiredDataError('initialize', object='testing')

    # Test raising the UnknownFileTypeError exception
    with pytest.raises(exceptions.UnknownFileTypeError):
        raise exceptions.UnknownFileTypeError()
    with pytest.raises(exceptions.UnknownFileTypeError):
        raise exceptions.UnknownFileTypeError(file='testing.txt')

    # Test raising the APIConnectionError exception
    with pytest.raises(exceptions.APIConnectionError):
        raise exceptions.APIConnectionError()

    # Test raising the APIRequestError exception
    with pytest.raises(exceptions.APIRequestError):
        raise exceptions.APIRequestError()

    # Test raising the DELETERequestError exception
    with pytest.raises(exceptions.DELETERequestError):
        raise exceptions.DELETERequestError()

    # Test raising the FeatureNotConfiguredError exception
    with pytest.raises(exceptions.FeatureNotConfiguredError):
        raise exceptions.FeatureNotConfiguredError()
    with pytest.raises(exceptions.FeatureNotConfiguredError):
        raise exceptions.FeatureNotConfiguredError(identifier='testing')
    with pytest.raises(exceptions.FeatureNotConfiguredError):
        raise exceptions.FeatureNotConfiguredError(feature='testing')

    # Test raising the GETRequestError exception
    with pytest.raises(exceptions.GETRequestError):
        raise exceptions.GETRequestError()
    with pytest.raises(exceptions.GETRequestError):
        raise exceptions.GETRequestError(status_code=400)
    with pytest.raises(exceptions.GETRequestError):
        raise exceptions.GETRequestError(message='testing')
    with pytest.raises(exceptions.GETRequestError):
        raise exceptions.GETRequestError(status_code=400, message='testing')

    # Test raising the InvalidEndpointError exception
    with pytest.raises(exceptions.InvalidEndpointError):
        raise exceptions.InvalidEndpointError()

    # Test raising the InvalidLookupTypeError exception
    with pytest.raises(exceptions.InvalidLookupTypeError):
        raise exceptions.InvalidLookupTypeError()

    # Test raising the InvalidPayloadValueError exception
    with pytest.raises(exceptions.InvalidPayloadValueError):
        raise exceptions.InvalidPayloadValueError()
    with pytest.raises(exceptions.InvalidPayloadValueError):
        raise exceptions.InvalidPayloadValueError(value='test_value')
    with pytest.raises(exceptions.InvalidPayloadValueError):
        raise exceptions.InvalidPayloadValueError(value='test_value', field='test_field')

    # Test raising the InvalidRequestTypeError exception
    with pytest.raises(exceptions.InvalidRequestTypeError):
        raise exceptions.InvalidRequestTypeError()

    # Test raising the LookupMismatchError exception
    with pytest.raises(exceptions.LookupMismatchError):
        raise exceptions.LookupMismatchError()

    # Test raising the NotFoundResponseError exception
    with pytest.raises(exceptions.NotFoundResponseError):
        raise exceptions.NotFoundResponseError()

    # Test raising the PayloadMismatchError exception
    with pytest.raises(exceptions.PayloadMismatchError):
        raise exceptions.PayloadMismatchError()
    with pytest.raises(exceptions.PayloadMismatchError):
        raise exceptions.PayloadMismatchError(request_type='post')

    # Test raising the POSTRequestError exception
    with pytest.raises(exceptions.POSTRequestError):
        raise exceptions.POSTRequestError()
    with pytest.raises(exceptions.POSTRequestError):
        raise exceptions.POSTRequestError(status_code=400)
    with pytest.raises(exceptions.POSTRequestError):
        raise exceptions.POSTRequestError(message='testing')
    with pytest.raises(exceptions.POSTRequestError):
        raise exceptions.POSTRequestError(status_code=400, message='testing')

    # Test raising the PUTRequestError exception
    with pytest.raises(exceptions.PUTRequestError):
        raise exceptions.PUTRequestError()
    with pytest.raises(exceptions.PUTRequestError):
        raise exceptions.PUTRequestError(status_code=400)
    with pytest.raises(exceptions.PUTRequestError):
        raise exceptions.PUTRequestError(message='testing')
    with pytest.raises(exceptions.PUTRequestError):
        raise exceptions.PUTRequestError(status_code=400, message='testing')

    # Test raising the InvalidHelperFileTypeError exception
    with pytest.raises(exceptions.InvalidHelperFileTypeError):
        raise exceptions.InvalidHelperFileTypeError()

    # Test raising the InvalidHelperArgumentsError exception
    with pytest.raises(exceptions.InvalidHelperArgumentsError):
        raise exceptions.InvalidHelperArgumentsError()

    # Test raising the HelperFunctionNotFoundError exception
    with pytest.raises(exceptions.HelperFunctionNotFoundError):
        raise exceptions.HelperFunctionNotFoundError()

    # Test raising the InvalidOperatorError exception
    with pytest.raises(exceptions.InvalidOperatorError):
        raise exceptions.InvalidOperatorError()

    # Test raising the LiQLParseError exception
    with pytest.raises(exceptions.LiQLParseError):
        raise exceptions.LiQLParseError()
    with pytest.raises(exceptions.LiQLParseError):
        raise exceptions.LiQLParseError(message='testing')

    # Test raising the OperatorMismatchError exception
    with pytest.raises(exceptions.OperatorMismatchError):
        raise exceptions.OperatorMismatchError()

    # Test raising the TooManyResultsError exception
    with pytest.raises(exceptions.TooManyResultsError):
        raise exceptions.TooManyResultsError()

    # Test raising the InvalidMetadataError exception
    with pytest.raises(exceptions.InvalidMetadataError):
        raise exceptions.InvalidMetadataError()
    with pytest.raises(exceptions.InvalidMetadataError):
        raise exceptions.InvalidMetadataError(metadata='testing')

    # Test raising the MessageTypeNotFoundError exception
    with pytest.raises(exceptions.MessageTypeNotFoundError):
        raise exceptions.MessageTypeNotFoundError()
    with pytest.raises(exceptions.MessageTypeNotFoundError):
        raise exceptions.MessageTypeNotFoundError(msg_type='testing')
    with pytest.raises(exceptions.MessageTypeNotFoundError):
        raise exceptions.MessageTypeNotFoundError(url='khoros.com')

    # Test raising the InvalidMessagePayloadError exception
    with pytest.raises(exceptions.InvalidMessagePayloadError):
        raise exceptions.InvalidMessagePayloadError()

    # Test raising the InvalidNodeTypeError exception
    with pytest.raises(exceptions.InvalidNodeTypeError):
        raise exceptions.InvalidNodeTypeError()
    with pytest.raises(exceptions.InvalidNodeTypeError):
        raise exceptions.InvalidNodeTypeError(val='testing')

    # Test raising the NodeIDNotFoundError exception
    with pytest.raises(exceptions.NodeIDNotFoundError):
        raise exceptions.NodeIDNotFoundError()
    with pytest.raises(exceptions.NodeIDNotFoundError):
        raise exceptions.NodeIDNotFoundError(val='testing')

    # Test raising the NodeTypeNotFoundError exception
    with pytest.raises(exceptions.NodeTypeNotFoundError):
        raise exceptions.NodeTypeNotFoundError()
    with pytest.raises(exceptions.NodeTypeNotFoundError):
        raise exceptions.NodeTypeNotFoundError(val='testing')

    # Test raising the UnsupportedNodeTypeError exception
    with pytest.raises(exceptions.UnsupportedNodeTypeError):
        raise exceptions.UnsupportedNodeTypeError()
    with pytest.raises(exceptions.UnsupportedNodeTypeError):
        raise exceptions.UnsupportedNodeTypeError(node_type='test_type')
    with pytest.raises(exceptions.UnsupportedNodeTypeError):
        raise exceptions.UnsupportedNodeTypeError(node_type='test_type', operation='testing stuff')

    # Test raising the InvalidRoleError exception
    with pytest.raises(exceptions.InvalidRoleError):
        raise exceptions.InvalidRoleError()
    with pytest.raises(exceptions.InvalidRoleError):
        raise exceptions.InvalidRoleError(role='testing')

    # Test raising the InvalidRoleTypeError exception
    with pytest.raises(exceptions.InvalidRoleTypeError):
        raise exceptions.InvalidRoleTypeError()
    with pytest.raises(exceptions.InvalidRoleTypeError):
        raise exceptions.InvalidRoleTypeError(role_type='testing')

    # Test raising the UserCreationError exception
    with pytest.raises(exceptions.UserCreationError):
        raise exceptions.UserCreationError()
    with pytest.raises(exceptions.UserCreationError):
        raise exceptions.UserCreationError(user='testing')
    with pytest.raises(exceptions.UserCreationError):
        raise exceptions.UserCreationError(exc_msg='this is an error message')



# -*- coding: utf-8 -*-
"""
:Module:            khoros.structures.boards
:Synopsis:          This module contains functions specific to boards within the Khoros Community platform
:Usage:             ``from khoros.structures import boards``
:Example:           ``board_url = boards.create(khoros_object, 'my-board', 'My Board', 'forum', return_url=True)``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     23 May 2022
"""

import warnings

from .. import api, errors
from ..objects import users
from ..utils import log_utils
from . import base

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)

VALID_DISCUSSION_STYLES = ['blog', 'contest', 'forum', 'idea', 'qanda', 'tkb']


def create(khoros_object, board_id, board_title, discussion_style, description=None, parent_category_id=None,
           hidden=None, allowed_labels=None, use_freeform_labels=None, use_predefined_labels=None,
           predefined_labels=None, media_type=None, blog_authors=None, blog_author_ids=None, blog_author_logins=None,
           blog_comments_enabled=None, blog_moderators=None, blog_moderator_ids=None, blog_moderator_logins=None,
           one_entry_per_contest=None, one_kudo_per_contest=None, posting_date_end=None, posting_date_start=None,
           voting_date_end=None, voting_date_start=None, winner_announced_date=None, full_response=None, return_id=None,
           return_url=None, return_api_url=None, return_http_code=None, return_status=None, return_error_messages=None,
           split_errors=False):
    """This function creates a new board within a Khoros Community environment.

    .. versionchanged:: 2.5.2
       Changed the functionality around the ``return_error_messages`` argument and added the ``split_errors`` argument.

    .. versionadded:: 2.5.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param board_id: The unique identifier (i.e. ``id`` field) for the new board **(Required)**
    :type board_id: str
    :param board_title: The title of the new board **(Required)**
    :type board_title: str
    :param discussion_style: Defines the board as a ``blog``, ``contest``, ``forum``, ``idea``, ``qanda`` or ``tkb``
                             **(Required)**
    :type discussion_style: str
    :param description: A brief description of the board
    :type description: str, None
    :param parent_category_id: The ID of the parent category (if applicable)
    :type parent_category_id: str, None
    :param hidden: Defines whether or not the new board should be hidden from lists and menus (disabled by default)
    :type hidden: bool, None
    :param allowed_labels: The type of labels allowed on the board (``freeform-only``, ``predefined-only`` or
                           ``freeform and pre-defined``)
    :type allowed_labels: str, None
    :param use_freeform_labels: Determines if freeform labels should be utilized
    :type use_freeform_labels: bool, None
    :param use_predefined_labels: Determines if pre-defined labels should be utilized
    :type use_predefined_labels: bool, None
    :param predefined_labels: The pre-defined labels to utilized on the board as a list of dictionaries

                              .. todo:: The ability to provide labels as a simple list and optionally standardize
                                        their format (e.g. Pascal Case, etc.) will be available in a future release.

    :type predefined_labels: list, None
    :param media_type: The media type associated with a contest. (``image``, ``video`` or ``story`` meaning text)
    :type media_type: str, None
    :param blog_authors: The approved blog authors in a blog board as a list of user data dictionaries
    :type blog_authors: list, None
    :param blog_author_ids: A list of User IDs representing the approved blog authors in a blog board
    :type blog_author_ids: list, None
    :param blog_author_logins: A list of User Logins (i.e. usernames) representing approved blog authors in a blog board
    :type blog_author_logins: list, None
    :param blog_comments_enabled: Determines if comments should be enabled on blog posts within a blog board
    :type blog_comments_enabled: bool, None
    :param blog_moderators: The designated blog moderators in a blog board as a list of user data dictionaries
    :type blog_moderators: list, None
    :param blog_moderator_ids: A list of User IDs representing the blog moderators in a blog board
    :type blog_moderator_ids: list, None
    :param blog_moderator_logins: A list of User Logins (i.e. usernames) representing blog moderators in a blog board
    :type blog_moderator_logins: list, None
    :param one_entry_per_contest: Defines whether a user can submit only one entry to a single contest
    :type one_entry_per_contest: bool, None
    :param one_kudo_per_contest: Defines whether a user can vote only once per contest
    :type one_kudo_per_contest: bool, None
    :param posting_date_end: The date/time when the contest is closed to submissions
    :type posting_date_end: type[datetime.datetime], None
    :param posting_date_start: The date/time when the voting period for a contest begins
    :type posting_date_start: type[datetime.datetime], None
    :param voting_date_end: The date/time when the voting period for a contest ends
    :type voting_date_end: type[datetime.datetime], None
    :param voting_date_start: The date/time when the voting period for a contest begins
    :type voting_date_start: type[datetime.datetime], None
    :param winner_announced_date: The date/time the contest winner will be announced
    :type winner_announced_date: type[datetime.datetime], None
    :param full_response: Determines whether the full, raw API response should be returned by the function

                          .. caution:: This argument overwrites the ``return_id``, ``return_url``, ``return_api_url``,
                                       ``return_http_code``, ``return_status`` and ``return_error_messages`` arguments.

    :type full_response: bool, None
    :param return_id: Determines if the **ID** of the new board should be returned by the function
    :type return_id: bool, None
    :param return_url: Determines if the **URL** of the new board should be returned by the function
    :type return_url: bool, None
    :param return_api_url: Determines if the **API URL** of the new board should be returned by the function
    :type return_api_url: bool, None
    :param return_http_code: Determines if the **HTTP Code** of the API response should be returned by the function
    :type return_http_code: bool, None
    :param return_status: Determines if the **Status** of the API response should be returned by the function
    :type return_status: bool, None
    :param return_error_messages: Determines if the **Developer Response Message** (if any) associated with the
           API response should be returned by the function
    :type return_error_messages: bool, None
    :param split_errors: Defines whether or not error messages should be merged when applicable
    :type split_errors: bool
    :returns: Boolean value indicating a successful outcome (default), the full API response or one or more specific
              fields defined by function arguments
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`,
             :py:exc:`khoros.errors.exceptions.InvalidNodeTypeError`,
             :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.APIConnectionError`,
             :py:exc:`khoros.errors.exceptions.POSTRequestError`
    """
    payload = structure_payload(khoros_object, board_id, board_title, discussion_style, description, parent_category_id,
                                hidden, allowed_labels, use_freeform_labels, use_predefined_labels, predefined_labels,
                                media_type, blog_authors, blog_author_ids, blog_author_logins, blog_comments_enabled,
                                blog_moderators, blog_moderator_ids, blog_moderator_logins, one_entry_per_contest,
                                one_kudo_per_contest, posting_date_end, posting_date_start, voting_date_end,
                                voting_date_start, winner_announced_date)
    # TODO: Add a new api.make_v2_request() function that just takes an endpoint rather than the full URL
    api_url = f"{khoros_object.core['v2_base']}/boards"
    headers = {'content-type': 'application/json'}
    response = api.post_request_with_retries(api_url, payload, khoros_object=khoros_object, headers=headers)
    return api.deliver_v2_results(response, full_response, return_id, return_url, return_api_url, return_http_code,
                                  return_status, return_error_messages, split_errors)


def structure_payload(khoros_object, board_id, board_title, discussion_style, description=None, parent_category_id=None,
                      hidden=None, allowed_labels=None, use_freeform_labels=None, use_predefined_labels=None,
                      predefined_labels=None, media_type=None, blog_authors=None, blog_author_ids=None,
                      blog_author_logins=None, blog_comments_enabled=None, blog_moderators=None,
                      blog_moderator_ids=None, blog_moderator_logins=None, one_entry_per_contest=None,
                      one_kudo_per_contest=None, posting_date_end=None, posting_date_start=None, voting_date_end=None,
                      voting_date_start=None, winner_announced_date=None):
    """This function structures the payload to use in a Community API v2 request involving a board.

    .. versionadded:: 2.5.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param board_id: The unique identifier (i.e. ``id`` field) for the new board **(Required)**
    :type board_id: str
    :param board_title: The title of the new board **(Required)**
    :type board_title: str
    :param discussion_style: Defines the board as a ``blog``, ``contest``, ``forum``, ``idea``, ``qanda`` or ``tkb``
                             **(Required)**
    :type discussion_style: str
    :param description: A brief description of the board
    :type description: str, None
    :param parent_category_id: The ID of the parent category (if applicable)
    :type parent_category_id: str, None
    :param hidden: Defines whether or not the new board should be hidden from lists and menus (disabled by default)
    :type hidden: bool, None
    :param allowed_labels: The type of labels allowed on the board (``freeform-only``, ``predefined-only`` or
                           ``freeform and pre-defined``)
    :type allowed_labels: str, None
    :param use_freeform_labels: Determines if freeform labels should be utilized
    :type use_freeform_labels: bool, None
    :param use_predefined_labels: Determines if pre-defined labels should be utilized
    :type use_predefined_labels: bool, None
    :param predefined_labels: The pre-defined labels to utilized on the board as a list of dictionaries

                              .. todo:: The ability to provide labels as a simple list and optionally standardize
                                        their format (e.g. Pascal Case, etc.) will be available in a future release.

    :type predefined_labels: list, None
    :param media_type: The media type associated with a contest. (``image``, ``video`` or ``story`` meaning text)
    :type media_type: str, None
    :param blog_authors: The approved blog authors in a blog board as a list of user data dictionaries
    :type blog_authors: list, None
    :param blog_author_ids: A list of User IDs representing the approved blog authors in a blog board
    :type blog_author_ids: list, None
    :param blog_author_logins: A list of User Logins (i.e. usernames) representing approved blog authors in a blog board
    :type blog_author_logins: list, None
    :param blog_comments_enabled: Determines if comments should be enabled on blog posts within a blog board
    :type blog_comments_enabled: bool, None
    :param blog_moderators: The designated blog moderators in a blog board as a list of user data dictionaries
    :type blog_moderators: list, None
    :param blog_moderator_ids: A list of User IDs representing the blog moderators in a blog board
    :type blog_moderator_ids: list, None
    :param blog_moderator_logins: A list of User Logins (i.e. usernames) representing blog moderators in a blog board
    :type blog_moderator_logins: list, None
    :param one_entry_per_contest: Defines whether a user can submit only one entry to a single contest
    :type one_entry_per_contest: bool, None
    :param one_kudo_per_contest: Defines whether a user can vote only once per contest
    :type one_kudo_per_contest: bool, None
    :param posting_date_end: The date/time when the contest is closed to submissions
    :type posting_date_end: type[datetime.datetime], None
    :param posting_date_start: The date/time when the voting period for a contest begins
    :type posting_date_start: type[datetime.datetime], None
    :param voting_date_end: The date/time when the voting period for a contest ends
    :type voting_date_end: type[datetime.datetime], None
    :param voting_date_start: The date/time when the voting period for a contest begins
    :type voting_date_start: type[datetime.datetime], None
    :param winner_announced_date: The date/time the contest winner will be announced
    :type winner_announced_date: type[datetime.datetime], None
    :returns: The full and properly formatted payload for the API request
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`,
             :py:exc:`khoros.errors.exceptions.InvalidNodeTypeError`
    """
    # Define the minimum payload framework
    payload = {
        "data": {
            "type": "board"
        }
    }

    # Populate relevant sections of the payload
    required_error_msg = "The 'board_id', 'board_title' and 'discussion_style' fields are required to create a board."
    payload = _structure_id_and_title(board_id, board_title, payload, required_error_msg)
    payload = _structure_discussion_style(discussion_style, payload, required_error_msg)
    payload = _structure_parent_category(parent_category_id, payload)

    # Populate the remaining simple fields
    simple_fields = {
        'description': description,
        'hidden': hidden,
        'media_type': media_type,
    }
    payload = _structure_simple_fields(simple_fields, payload)

    # Populate the label settings
    label_settings = {
        'allowed_labels': allowed_labels,
        'use_freeform_labels': use_freeform_labels,
        'use_predefined_labels': use_predefined_labels,
        'predefined_labels': predefined_labels,
    }
    payload = _structure_label_settings(label_settings, payload)

    # Populate the blog-related settings
    blog_settings = {
        'authors': blog_authors,
        'author_ids': blog_author_ids,
        'author_logins': blog_author_logins,
        'comments_enabled': blog_comments_enabled,
        'moderators': blog_moderators,
        'moderator_ids': blog_moderator_ids,
        'moderator_logins': blog_moderator_logins,
    }
    payload = _structure_blog_settings(khoros_object, blog_settings, payload, discussion_style)

    # Populate the contest-related settings
    contest_settings = {
        'one_entry_per_contest': one_entry_per_contest,
        'one_kudo_per_contest': one_kudo_per_contest,
        'posting_date_end': posting_date_end,
        'posting_date_start': posting_date_start,
        'voting_date_end': voting_date_end,
        'voting_date_start': voting_date_start,
        'winner_announced_date': winner_announced_date,
    }
    payload = _structure_contest_settings(contest_settings, payload, discussion_style)
    return payload


def _structure_id_and_title(_board_id, _board_title, _payload, _missing_error):
    """This function structures the portion of the payload for the Board ID and Board Title.

    .. versionadded:: 2.5.0

    :param _board_id: The unique identifier (i.e. ``id`` field) for the new board **(Required)**
    :type _board_id: str
    :param _board_title: The title of the new board **(Required)**
    :type _board_title: str
    :param _payload: The partially constructed payload for the API request
    :type _payload: dict
    :param _missing_error: The error message to use when raising the
                           :py:exc:`khoros.errors.exceptions.MissingRequiredDataError` exception class when the
                           Board ID and/or Board Title have not been provided.
    :returns: The API request payload with appended entries for ``id`` and ``title``
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    if not _board_id or not _board_title:
        raise errors.exceptions.MissingRequiredDataError(_missing_error)
    _payload['data']['id'] = _board_id
    _payload['data']['title'] = _board_title
    return _payload


def _structure_discussion_style(_discussion_style, _payload, _missing_error):
    """This function structures the portion of the payload for the Discussion Style. (i.e. board type)

    .. versionadded:: 2.5.0

    :param _discussion_style: Defines the board as a ``blog``, ``contest``, ``forum``, ``idea``, ``qanda`` or ``tkb``
                              **(Required)**
    :type _discussion_style: str
    :param _payload: The partially constructed payload for the API request
    :type _payload: dict
    :param _missing_error: The error message to use when raising the
                           :py:exc:`khoros.errors.exceptions.MissingRequiredDataError` exception class when the
                           Discussion Style have not been provided.
    :returns: The API request payload with appended entries for ``conversation_style``
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`,
             :py:exc:`khoros.errors.exceptions.InvalidNodeTypeError`
    """
    if not _discussion_style:
        raise errors.exceptions.MissingRequiredDataError(_missing_error)
    if _discussion_style not in VALID_DISCUSSION_STYLES:
        raise errors.exceptions.InvalidNodeTypeError(f"'{_discussion_style}' is not a valid discussion style.")
    _payload['data']['conversation_style'] = _discussion_style
    return _payload


def _structure_parent_category(_parent_id, _payload):
    """This function structures the portion of the payload for the parent category.

    .. versionadded:: 2.5.0

    :param _parent_id: The ID of the parent category (if applicable)
    :type _parent_id: str, None
    :param _payload: The partially constructed payload for the API request
    :type _payload: dict
    :returns: The API request payload with appended entries for ``parent_category``
    """
    if _parent_id:
        _payload['data']['parent_category'] = {
            'id': _parent_id
        }
    return _payload


def _structure_simple_fields(_simple_fields, _payload):
    """This function structures the portion of the payload for the simple string and Boolean fields.

    .. versionadded:: 2.5.0

    :param _simple_fields: A dictionary containing the simple string and Boolean fields for the API payload
    :type _simple_fields: dict
    :param _payload: The partially constructed payload for the API request
    :type _payload: dict
    :returns: The API request payload with appended entries for the simple string and Boolean fields
    """
    for _field, _value in _simple_fields.items():
        if _value:
            _payload['data'][_field] = _value
    return _payload


def _structure_label_settings(_label_settings, _payload):
    """This function structures the portion of the payload for the setting fields relating to labels.

    .. versionadded:: 2.5.0

    :param _label_settings: A dictionary containing the settings involving labels
    :type _label_settings: dict
    :param _payload: The partially constructed payload for the API request
    :type _payload: dict
    :returns: The API request payload with appended entries for the field settings relating to labels
    """
    # Verify that any string passed in 'allowed_labels' is valid when applicable
    _valid_allowed_labels = ['freeform-only', 'predefined-only', 'freeform and pre-defined']
    if _label_settings.get('allowed_labels'):
        if _label_settings.get('allowed_labels') in _valid_allowed_labels:
            _payload['data']['allowed_labels'] = _label_settings.get('allowed_labels')
        else:
            # TODO: Leverage the logger instead of warnings
            warn_msg = f"The string '{_label_settings.get('allowed_labels')}' for the 'allowed_labels' field is " \
                       f"not valid and will be ignored."
            warnings.warn(warn_msg, UserWarning)
            _label_settings['allowed_labels'] = None

    # Inform the user that the value will be overwritten by any defined Boolean label settings values
    if _label_settings.get('allowed_labels') and (
            _label_settings.get('allowed_labels') is not None or
            _label_settings.get('allowed_labels') is not None):
        # TODO: Leverage the logger instead of warnings
        warn_msg = f"The defined 'allowed_labels' field will be overwritten when the 'use_freeform_labels' and/or " \
                   f"'use_predefined_labels' Boolean values are also configured."
        warnings.warn(warn_msg, UserWarning)

    # Define the 'allowed_labels' value based on the defined Boolean values when applicable
    _boolean_values = (_label_settings.get('use_freeform_labels'), _label_settings.get('use_predefined_labels'))
    if any(_boolean_values):
        if all(_boolean_values):
            _payload['data']['allowed_labels'] = 'freeform and pre-defined'
        elif _label_settings.get('use_freeform_labels'):
            _payload['data']['allowed_labels'] = 'freeform-only'
        elif _label_settings.get('use_predefined_labels'):
            _payload['data']['allowed_labels'] = 'predefined-only'

    # Add the predefined labels if present
    # TODO: Check to ensure that they are in the proper format (In Studio they are a comma-separated list)
    # TODO: Provide the option to format the labels into all lowercase or Pascal Case
    # TODO: Set up unit test to see what happens if 'use_predefined_labels' is True but no labels are predefined
    if _label_settings.get('predefined_labels'):
        _payload['data']['predefined_labels'] = _label_settings.get('predefined_labels')
    return _payload


def _structure_blog_settings(_khoros_object, _blog_settings, _payload, _discussion_style):
    """This function structures the portion of the payload for the setting fields relating to blogs.

    .. versionadded:: 2.5.0

    :param _khoros_object:
    :param _blog_settings: A dictionary containing the settings involving labels
    :type _blog_settings: dict
    :param _payload: The partially constructed payload for the API request
    :type _payload: dict
    :param _discussion_style: The discussion style for the new board to ensure that these settings apply
    :type _discussion_style: str
    :returns: The API request payload with appended entries for the field settings relating to labels
    """
    if any(_blog_settings.values()) and _discussion_style != 'blog':
        _warn_about_ignored_settings('blog', _discussion_style)
    else:
        # Populate the 'comments_enabled' setting if applicable
        if _blog_settings.get('comments_enabled'):
            _payload['data']['comments_enabled'] = _blog_settings.get('comments_enabled')

        # Populate the blog authors list
        if any((_blog_settings['authors'], _blog_settings['author_ids'], _blog_settings['author_logins'])):
            authors = users.structure_user_dict_list(_khoros_object, _blog_settings['authors'],
                                                     _blog_settings['author_ids'], _blog_settings['author_logins'])
            _payload['data']['authors'] = authors

        # Populate the blog moderators list
        if any((_blog_settings['moderators'], _blog_settings['moderator_ids'], _blog_settings['moderator_logins'])):
            moderators = users.structure_user_dict_list(_khoros_object, _blog_settings['moderators'],
                                                        _blog_settings['moderator_ids'],
                                                        _blog_settings['moderator_logins'])
            _payload['data']['moderators'] = moderators
    return _payload


def _structure_contest_settings(_contest_settings, _payload, _discussion_style):
    """This function structures the portion of the payload for the setting fields relating to contests.

    .. versionadded:: 2.5.0

    :param _contest_settings: A dictionary containing the settings involving contests
    :type _contest_settings: dict
    :param _payload: The partially constructed payload for the API request
    :type _payload: dict
    :param _discussion_style: The discussion style for the new board to ensure that these settings apply
    :type _discussion_style: str
    :returns: The API request payload with appended entries for the field settings relating to contests
    """
    if any(_contest_settings.values()) and _discussion_style != 'contest':
        _warn_about_ignored_settings('contest', _discussion_style)
    else:
        for _field, _value in _contest_settings.items():
            if _value:
                # TODO: Verify proper format for datetime values
                _payload['data'][_field] = _value
    return _payload


def _warn_about_ignored_settings(_settings_type, _discussion_style):
    """This function displays a ``UserWarning`` that provided fields will be ignored if the discussion style does not
       match the style for which the field are specific.

    .. versionchanged:: 5.0.0
       Removed the redundant return statement.

    .. versionadded:: 2.5.0

    :param _settings_type: The discussion style relating to the supplied fields and values
    :type _settings_type: str
    :param _discussion_style: The discussion style of the new board
    :type _discussion_style: str
    :returns: None
    """
    # TODO: Leverage the logger instead of warnings
    warn_msg = f"Because the discussion style is '{_discussion_style}' all {_settings_type}-specific fields " \
               "provided will be ignored."
    warnings.warn(warn_msg, UserWarning)


def get_board_id(url):
    """This function retrieves the Board ID for a given board when provided its URL.

    .. versionadded:: 2.6.0

    :param url: The URL from which to parse out the Board ID
    :type url: str
    :returns: The Board ID retrieved from the URL
    :raises: :py:exc:`khoros.errors.exceptions.InvalidURLError`
    """
    return base.get_structure_id(url)


def board_exists(khoros_object, board_id=None, board_url=None):
    """This function checks to see if a board (i.e. blog, contest, forum, idea exchange, Q&A or TKB) exists.

    .. versionadded:: 2.7.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param board_id: The ID of the board to check
    :type board_id: str, None
    :param board_url: The URL of the board to check
    :type board_url: str, None
    :returns: Boolean value indicating whether or not the board already exists
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return base.structure_exists(khoros_object, 'board', board_id, board_url)



# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.tests.test_version
:Synopsis:          This module is used by pytest to verify that the version module functions correctly
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     26 Sep 2022
"""

import pytest

from . import resources


def test_full_version():
    """This function tests to verify that the full version is defined correctly.

    .. versionadded:: 5.1.0
    """
    assert khoros_object.version == version.get_full_version()


def test_major_minor_version():
    """This function tests to ensure that the major/minor version is getting defined correctly.

    .. versionadded:: 5.1.0
    """
    major_minor = ".".join(khoros_object.version.split(".")[:2])
    assert major_minor == version.get_major_minor_version()


def test_latest_stable():
    """This function tests to ensure that the latest stable version can be retrieved successfully.

    .. versionadded:: 5.1.0
    """
    latest_stable = version.get_latest_stable()
    assert latest_stable != '0.0.0'


def test_latest_version():
    """This function tests to ensure that the check to see if the version is the latest stable works properly.

    .. versionadded:: 5.1.0
    """
    is_latest = version.latest_version()
    assert isinstance(is_latest, bool)


# Initialize the core object
version, exceptions = resources.import_modules('khoros.utils.version', 'khoros.errors.exceptions')
khoros_object = resources.initialize_khoros_object()


# -*- coding: utf-8 -*-
"""
:Module:            khoros.studio.base
:Synopsis:          This module handles the base functionality of the studio module.
:Usage:             ``import khoros.studio.base as studio_base``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     17 Jul 2020
"""

from ..utils import core_utils, log_utils

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)


def sdk_installed():
    """This function checks to see if the Lithium SDK is installed.

    .. versionadded:: 2.5.1

    :returns: Boolean value indicating whether or not the Lithium SDK is installed
    """
    try:
        output = core_utils.run_cmd('li')
        is_installed = True if output['return_code'] == 0 else False
    except FileNotFoundError:
        is_installed = False
    return is_installed


def get_sdk_version():
    """This function identifies the currently installed version of the Lithium SDK.

    .. versionadded:: 2.5.1

    :returns: The SDK version in string format or ``None`` if not installed
    """
    version = None
    if npm_installed() and sdk_installed():
        output = core_utils.run_cmd('npm list -g lithium-sdk', decode_output=True, strip_output=False)['stdout']
        output = core_utils.decode_binary(output)
        if '(empty)' not in output:
            version = output.split('lithium-sdk@')[1].split(' ')[0]
    return version


def node_installed():
    """This function checks whether or not Node.js is installed.

    .. versionadded:: 2.5.1

    :returns: Boolean value indicating whether or not Node.js is installed
    """
    node_version = get_node_version()
    return True if node_version else False


def get_node_version():
    """This function identifies and returns the installed Node.js version.

    .. versionadded:: 2.5.1

    :returns: The version as a string or ``None`` if not installed
    """
    try:
        version = core_utils.run_cmd('node -v', decode_output=True, strip_output=True)['stdout']
        if 'v' in version:
            version = version[1:]
    except FileNotFoundError:
        version = None
    return version


def npm_installed():
    """This function checks whether or not npm is installed.

    .. versionadded:: 2.5.1

    :returns: Boolean value indicating whether or not npm is installed
    """
    npm_version = get_npm_version()
    return True if npm_version else False


def get_npm_version():
    """This function identifies and returns the installed npm version.

    .. versionadded:: 2.5.1

    :returns: The version as a string or ``None`` if not installed
    """
    try:
        version = core_utils.run_cmd('npm -v', decode_output=True, strip_output=True)['stdout']
        if 'v' in version:
            version = version[1:]
    except FileNotFoundError:
        version = None
    return version



# -*- coding: utf-8 -*-
"""
:Module:            khoros.studio
:Synopsis:          This module performs tasks that relate to the Lithium SDK and the Khoros Community Studio Plugin.
:Usage:             ``from khoros import studio``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     20 May 2020
"""

__all__ = ['base']

# Import all submodules by default
from . import base



# -*- coding: utf-8 -*-
"""
:Module:            khoros.objects.saml
:Synopsis:          This module includes functions that relate to SAML SSO.
:Usage:             ``from khoros import saml``
:Example:           ``assertion = saml.import_assertion(file_path)``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     23 May 2022
"""

from . import api, errors
from .utils import log_utils, core_utils

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)


def send_assertion(khoros_object, assertion=None, file_path=None, base64_encode=True, url_encode=True):
    """This function sends a SAML assertion as a POST request in order to provision a new user.

    .. versionadded:: 4.3.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param assertion: The SAML assertion in string format and optionally base64- and/or URL-encoded
    :type assertion: str, None
    :param file_path: The file path to the XML file to import that contains the SAML assertion
    :type file_path: str, None
    :param base64_encode: Determines if the assertion should be base64-encoded (``True`` by default)
    :type base64_encode: bool
    :param url_encode: Determines if the assertion should be URL-encoded (``True`` by default)
    :type url_encode: bool
    :returns: The API response from the POST request
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    if not any((assertion, file_path)):
        error_msg = "A SAML assertion string or file path must be provided"
        raise errors.exceptions.MissingRequiredDataError(error_msg)
    if assertion:
        if _is_decoded(assertion) and base64_encode:
            assertion = core_utils.encode_base64(assertion, url_encode_object=url_encode)
        elif not _is_decoded(assertion) and '=' in assertion and url_encode:
            assertion = core_utils.url_encode(assertion)
    else:
        assertion = import_assertion(file_path, base64_encode, url_encode)
    uri = _get_api_uri(khoros_object)
    response = api.post_request_with_retries(uri, url_encoded_payload=assertion, return_json=False,
                                             khoros_object=khoros_object)
    return response


def import_assertion(file_path, base64_encode=True, url_encode=True):
    """This function imports an XML SAML assertion as a string and optionally base64- and/or URL-encodes it.

    .. versionadded:: 4.3.0

    :param file_path: The file path to the XML file to import
    :type file_path: str
    :param base64_encode: Determines if the assertion should be base64-encoded (``True`` by default)
    :type base64_encode: bool
    :param url_encode: Determines if the assertion should be URL-encoded (``True`` by default)
    :type url_encode: bool
    :returns: The SAML assertion string
    :raises: :py:exc:`FileNotFoundError`
    """
    with open(file_path, 'r') as file:
        assertion = file.read()
    if base64_encode:
        assertion = core_utils.encode_base64(assertion)
    if url_encode:
        assertion = core_utils.url_encode(assertion)
    return assertion


def _is_decoded(_assertion):
    """This function checks to see if a SAML assertion is base64- or url-encoded.

    .. versionadded:: 4.3.0

    :param _assertion: The SAML assertion string to be examined
    :type _assertion: str
    :returns: Boolean value indicating whether the SAML assertion string is encoded
    :raises: :py:exc:`TypeError`
    """
    return True if '<saml' in _assertion else False


def _get_api_uri(_khoros_object):
    """This function retrieves the API URI to call in the POST request.

    .. versionadded:: 4.3.0

    :param _khoros_object: The core :py:class:`khoros.Khoros` object
    :type _khoros_object: class[khoros.Khoros]
    :returns: The absolute URI to be used in the POST request
    """
    _base_url = _khoros_object.core.get('base_url')
    _base_url = _base_url[:-1] if _base_url.endswith('/') else _base_url
    return _base_url + '/auth/saml/login?SAMLResponse'



# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.tests.test_albums
:Synopsis:          This module is used by pytest to verify that the ``albums`` module functions properly
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     30 Sep 2022
"""

import os
import sys

import requests

from . import resources

# Define a global variable to define when the package path has been set
package_path_defined = False


def set_package_path():
    """This function adds the high-level khoros directory to the sys.path list.

    .. versionadded:: 5.1.2
    """
    global package_path_defined
    if not package_path_defined:
        sys.path.insert(0, os.path.abspath('../..'))
        package_path_defined = True


def test_create_album(monkeypatch):
    """This function tests the ability to create an album.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Overwrite the requests.get functionality with the mock_post() function
    monkeypatch.setattr(requests, 'post', resources.mock_success_post)

    # Perform and assess the mock API call
    response = khoros_object.albums.create('My New Album', 'The description', full_response=False)
    assert response is True


def test_failed_create_album(monkeypatch):
    """This function tests the response of the ``create`` method when the API returns an error.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Overwrite the requests.get functionality with the mock_post() function
    monkeypatch.setattr(requests, 'post', resources.mock_error_post)

    # Perform and assess the mock API call
    response = khoros_object.albums.create('My New Album', 'The description', full_response=False)
    assert response is False



# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.tests.test_error_handling
:Synopsis:          This module is used by pytest to verify that error handling works properly
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     03 Oct 2022
"""

import pytest

from . import resources
from ...errors import handlers, exceptions, translations


def test_eprint():
    """This function tests printing an error message to the stderr output.

    .. versionadded:: 5.1.2
    """
    handlers.eprint('This is an error message')


def test_get_html_error():
    """This function tests the ability to parse an HTML error to get an error message.

    .. versionadded:: 5.1.2
    """
    # TODO: Add a true error message from the API
    html_error = "<html><body><h1>404 Not Found</h1><b>Error description</b><u></u></body></html>"
    v1_error = 'The Community API v1 call failed with the following error:\n\t404 Not Found'
    assert handlers.get_error_from_html(html_error) == '404 Not Found'
    assert handlers.get_error_from_html(html_error, v1=True) == v1_error


def test_verify_core_object_present():
    """This function tests the ability to verify that a Khoros core object is present.

    .. versionadded:: 5.1.2
    """
    with pytest.raises(exceptions.MissingRequiredDataError):
        khoros_object = None
        handlers.verify_core_object_present(khoros_object)


def test_translate_error():
    """This function tests the ability to translate obscure messages.

    .. versionadded:: 5.1.2
    """
    error_msg = 'page.post.error.attachment_bad_extension'
    translated_error_excerpt = 'The attachment does not have an extension permitted'
    assert translated_error_excerpt in translations.translate_error(error_msg)


def test_translation_check():
    """This function tests the ability to check if error translation is enabled.

    .. versionadded:: 5.1.2
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Call the function and verify that it is a Boolean response
    setting = translations.translation_enabled(khoros_object=khoros_object)
    assert isinstance(setting, bool)


def test_parse_translation_error():
    """This function tests the ability to parse an error message to remove extraneous escape characters.

    .. versionadded:: 5.1.2
    """
    error_msg_with_escape_char = 'This is an error message\n'
    error_msg = 'This is an error message'
    parsed_msg = translations.parse_message(error_msg_with_escape_char)
    assert parsed_msg == error_msg



# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.tests.test_roles
:Synopsis:          This module is used by pytest to verify the :py:mod:`khoros.objects.roles` functionality.
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     03 Oct 2022
"""

import os
import sys

import pytest

from . import resources
from ...errors import exceptions

# Define a global variable to define when the package path has been set
package_path_defined = False

# Import the roles module
roles = resources.import_modules('khoros.objects.roles')


def set_package_path():
    """This function adds the high-level khoros directory to the sys.path list.

    .. versionadded:: 5.0.0
    """
    global package_path_defined
    if not package_path_defined:
        sys.path.insert(0, os.path.abspath('../..'))
        package_path_defined = True


def test_get_role_id():
    """This function tests the :py:func:`khoros.objects.roles.get_role_id` function and corresponding method.

    .. versionchanged:: 5.1.2
       The function has been updated to support GitHub Workflows and to include a couple extra tests.

    .. versionadded:: 5.0.0
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Test the method and function
    role_id = khoros_object.roles.get_role_id('Administrator')
    assert role_id == 't:Administrator'                 # nosec
    role_id = khoros_object.roles.get_role_id('some_role', 'board', 'some-blog-node')
    assert role_id == 'b:some-blog-node:some_role'      # nosec

    # Test that the correct exception is raised for an invalid scope
    with pytest.raises(exceptions.InvalidRoleError):
        khoros_object.roles.get_role_id('Administrator', 'something_wrong')

    # Test that the correct exception is raised for a missing node ID
    with pytest.raises(exceptions.MissingRequiredDataError):
        khoros_object.roles.get_role_id('some_role', 'board')


def test_invalid_role_type():
    """This function tests passing an invalid role type to :py:func:`khoros.objects.roles.count_role_types`.

    .. versionadded:: 5.0.0
    """
    # Test that the appropriate exception is raised when an invalid role type is passed
    with pytest.raises(exceptions.InvalidRoleTypeError):
        roles.count_role_types('fake_role_type', {})


def test_total_role_type_counts():
    """This function tests the :py:meth:`khoros.core.Khoros.Role.get_total_role_count` method and related function.

    .. versionchanged:: 5.1.2
       The function has been updated to support GitHub Workflows.

    .. versionadded:: 5.0.0
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Ensure that the default result is an integer of the total count
    total_count = khoros_object.roles.get_total_role_count()
    assert isinstance(total_count, int)          # nosec

    # Ensure that adding another metric returns a tuple by default
    total_and_top_count = khoros_object.roles.get_total_role_count(top_level=True)
    assert isinstance(total_and_top_count, tuple)          # nosec

    # Ensure that disabling the total count and including another metric returns an integer
    only_top_count = khoros_object.roles.get_total_role_count(total=False, top_level=True)
    assert isinstance(only_top_count, int)          # nosec

    # Ensure that attempting to retrieve no metrics does not result in an exception
    no_metrics = khoros_object.roles.get_total_role_count(total=False)
    assert isinstance(no_metrics, tuple) and len(no_metrics) == 0          # nosec

    # Test the functionality to return a dictionary rather than a tuple
    counts_dict = khoros_object.roles.get_total_role_count(return_dict=True, top_level=True)
    assert isinstance(counts_dict, dict) and 'total' in counts_dict and 'top_level' in counts_dict          # nosec


def test_get_roles_for_user():
    """This function tests the :py:meth:`khoros.core.Khoros.Role.get_roles_for_user` method and related function.

    .. versionchanged:: 5.1.2
       The function has been updated to support GitHub Workflows.

    .. versionadded:: 5.0.0
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Test the method and function using an integer and a string as the User ID
    for user_id in [1, '1']:
        roles_for_user = khoros_object.roles.get_roles_for_user(user_id, 'id')
        assert isinstance(roles_for_user, list)          # nosec
        if len(roles_for_user) > 0:
            assert isinstance(roles_for_user[0], dict) and 'id' in roles_for_user[0]          # nosec
            assert 'href' not in roles_for_user[0]          # nosec


def test_get_users_with_role():
    """This function tests the :py:meth:`khoros.core.Khoros.Role.get_users_with_role` method and related function.

    .. versionchanged:: 5.1.2
       The function has been updated to support GitHub Workflows.

    .. versionadded:: 5.0.0
    """
    # Instantiate the core object
    khoros_object = resources.get_core_object()

    # Test the standard return mode
    users_with_role = khoros_object.roles.get_users_with_role(role_name='Administrator')
    assert isinstance(users_with_role, list)          # nosec
    if len(users_with_role) > 0:
        assert isinstance(users_with_role[0], dict) and 'login' in users_with_role[0]          # nosec

    # Test the simple return mode
    users_with_role = khoros_object.roles.get_users_with_role(role_name='Administrator', simple=True)
    assert isinstance(users_with_role, list)                # nosec
    if len(users_with_role) > 0:
        assert isinstance(users_with_role[0], str)          # nosec



# -*- coding: utf-8 -*-
"""
:Module:            khoros.objects.sub
:Synopsis:          This module includes functions that handle tags within a Khoros Community environment
:Usage:             ``from khoros.objects import subscriptions``
:Example:           ``response = subscribe_to_board(khoros_object, 'my-forum')``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     07 Apr 2021
"""

from .. import api, errors
from ..utils import core_utils, log_utils

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)


def add_subscription(khoros_object, target_id, target_type='board', payload=None, included_boards=None,
                     excluded_boards=None, label_board=None, proxy_user_object=None):
    """This function adds a subscription to a given target for the current user.

    .. versionchanged:: 4.0.0
       Introduced the ``proxy_user_object`` parameter to allow API requests to be performed on behalf of other users.

    .. versionadded:: 3.5.0

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :param target_id: The unique identifier for the target (e.g. Node ID, Message ID, etc.)
    :type target_id: str, int
    :param target_type: The target type such as ``board`` (default), ``message``, ``category``, etc.
    :param payload: Pre-constructed payload to use in the API call
    :type payload: dict, None
    :param included_boards: One or more boards (represented by Node ID) to be included in the partial subscription
    :type included_boards: list, tuple, set, str, None
    :param excluded_boards: One or more boards (represented by Node ID) to be excluded from the partial subscription
    :type excluded_boards: list, tuple, set, str, None
    :param label_board: The Board ID associated with a label (required for label subscriptions)
    :type label_board: str, int, None
    :param proxy_user_object: Instantiated :py:class:`khoros.objects.users.ImpersonatedUser` object to perform the
                              API request on behalf of a secondary user.
    :type proxy_user_object: class[khoros.objects.users.ImpersonatedUser], None
    :returns: The API response in JSON format
    :raises: :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.APIConnectionError`,
             :py:exc:`khoros.errors.exceptions.POSTRequestError`,
             :py:exc:`khoros.errors.exceptions.PayloadMismatchError`
    """
    if not payload:
        if target_type == 'category':
            payload = {"data": _construct_category_payload(target_id, included_boards, excluded_boards)}
        elif target_type == 'label':
            if not label_board:
                raise errors.exceptions.MissingRequiredDataError("A board ID is required when subscribing to labels")
            payload = {
                "data": _construct_target_subscription(target_id, target_type),
                "board": {"id": f"{label_board}"}
            }
        else:
            payload = {"data": _construct_target_subscription(target_id, target_type)}
    else:
        payload = {"data": payload} if not payload.get('data') else payload
    uri = get_subscription_uri(khoros_object)
    response = api.post_request_with_retries(uri, json_payload=payload, khoros_object=khoros_object,
                                             proxy_user_object=proxy_user_object)
    return response


def get_subscription_uri(khoros_object):
    """This function returns the subscriptions URI for the v2 API to perform API calls.

    .. versionadded:: 3.5.0

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :returns: The full (absolute) URI for the ``subscriptions`` v2 API endpoint
    """
    return f"{khoros_object.core_settings.get('v2_base')}/subscriptions"


def _construct_target_subscription(_target_id, _target_type='board'):
    """This function constructs a dictionary for an individual subscription target to be used in a payload.

    .. versionadded:: 3.5.0

    :param _target_id: The unique identifier for the target (e.g. Node ID)
    :type _target_id: str
    :param _target_type: The target type (``board`` by default)
    :type _target_type: str
    :returns: The dictionary for the individual target
    :raises: :py:exc:`TypeError`
    """
    _target = {
        "type": "subscription",
        "target": {
            "type": _target_type,
            "id": f"{_target_id}"
        }
    }
    return _target


# noinspection PyTypeChecker
def _construct_category_payload(_node_id, _included_boards=None, _excluded_boards=None):
    """This function constructs the payload for a full or partial category subscription.

    .. versionchanged:: 4.0.0
       Fixed an issue where the payload was getting double-wrapped with the ``data`` dictionary key.

    .. versionadded:: 3.5.0

    :param _node_id: The unique identifier (i.e. Node ID) of the category
    :type _node_id: str
    :param _included_boards: One or more boards (represented by Node ID) to be included in the partial subscription
    :type _included_boards: list, tuple, set, str, None
    :param _excluded_boards: One or more boards (represented by Node ID) to be excluded from the partial subscription
    :type _excluded_boards: list, tuple, set, str, None
    :returns: The payload to use in the API call as a dictionary
    :raises: :py:exc:`khoros.errors.exceptions.DataMismatchError`
    """
    # Construct the base data dictionary
    _payload = _construct_target_subscription(_node_id, 'category')

    # Ensure that only one populated partial option has been provided
    if _included_boards and _excluded_boards:
        raise errors.exceptions.DataMismatchError("Partial category subscriptions only support internal or external "
                                                  "boards but not both")

    # Added included or excluded boards when applicable
    _partials = {
        "includes": _included_boards,
        "excludes": _excluded_boards
    }
    for _partial_field, _boards in _partials.items():
        _boards = [_boards] if _boards and isinstance(_boards, str) else _boards
        if _boards and core_utils.is_iterable(_boards):
            _container = []
            for _board in _boards:
                _container.append(_construct_target_subscription(_board))
            _payload[_partial_field] = _container
    return _payload


def subscribe_to_board(khoros_object, node_id, proxy_user_object=None):
    """This function subscribes the current user to an individual message.

    .. versionchanged:: 4.0.0
       Introduced the ``proxy_user_object`` parameter to allow API requests to be performed on behalf of other users.

    .. versionadded:: 3.5.0

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :param node_id: The unique identifier for a board (i.e. Board ID)
    :type node_id: str
    :param proxy_user_object: Instantiated :py:class:`khoros.objects.users.ImpersonatedUser` object to perform the
                              API request on behalf of a secondary user.
    :type proxy_user_object: class[khoros.objects.users.ImpersonatedUser], None
    :returns: The API response in JSON format
    :raises: :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.APIConnectionError`,
             :py:exc:`khoros.errors.exceptions.POSTRequestError`,
             :py:exc:`khoros.errors.exceptions.PayloadMismatchError`
    """
    return add_subscription(khoros_object, node_id, 'board', proxy_user_object=proxy_user_object)


def subscribe_to_category(khoros_object, node_id, included_boards=None, excluded_boards=None, proxy_user_object=None):
    """This function subscribes the current user to a full or partial category.

    .. versionchanged:: 4.0.0
       Introduced the ``proxy_user_object`` parameter to allow API requests to be performed on behalf of other users.

    .. versionadded:: 3.5.0

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :param node_id: The unique identifier (i.e. Node ID) for the category
    :type node_id: str
    :param included_boards: One or more boards (represented by Node ID) to be included in the partial subscription
    :type included_boards: list, tuple, set, str, None
    :param excluded_boards: One or more boards (represented by Node ID) to be excluded from the partial subscription
    :type excluded_boards: list, tuple, set, str, None
    :param proxy_user_object: Instantiated :py:class:`khoros.objects.users.ImpersonatedUser` object to perform the
                              API request on behalf of a secondary user.
    :type proxy_user_object: class[khoros.objects.users.ImpersonatedUser], None
    :returns: The API response in JSON format
    :raises: :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.APIConnectionError`,
             :py:exc:`khoros.errors.exceptions.POSTRequestError`,
             :py:exc:`khoros.errors.exceptions.PayloadMismatchError`
    """
    return add_subscription(khoros_object, node_id, 'category',
                            included_boards=included_boards, excluded_boards=excluded_boards,
                            proxy_user_object=proxy_user_object)


def subscribe_to_label(khoros_object, label, board_id, proxy_user_object=None):
    """This function subscribes the current user to label found on a board.

    .. versionchanged:: 4.0.0
       Introduced the ``proxy_user_object`` parameter to allow API requests to be performed on behalf of other users.

    .. versionadded:: 3.5.0

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :param label: The label to which to subscribe
    :type label: str, int
    :param board_id: The unique identifier (i.e. Node ID) for the board where the label is found
    :type board_id: str
    :param proxy_user_object: Instantiated :py:class:`khoros.objects.users.ImpersonatedUser` object to perform the
                              API request on behalf of a secondary user.
    :type proxy_user_object: class[khoros.objects.users.ImpersonatedUser], None
    :returns: The API response in JSON format
    :raises: :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.APIConnectionError`,
             :py:exc:`khoros.errors.exceptions.POSTRequestError`,
             :py:exc:`khoros.errors.exceptions.PayloadMismatchError`
    """
    return add_subscription(khoros_object, label, 'label', label_board=board_id, proxy_user_object=proxy_user_object)


def subscribe_to_message(khoros_object, msg_id, proxy_user_object=None):
    """This function subscribes the current user to an individual message.

    .. versionchanged:: 4.0.0
       Introduced the ``proxy_user_object`` parameter to allow API requests to be performed on behalf of other users.

    .. versionadded:: 3.5.0

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :param msg_id: The unique identifier for an individual message
    :type msg_id: str, int
    :param proxy_user_object: Instantiated :py:class:`khoros.objects.users.ImpersonatedUser` object to perform the
                              API request on behalf of a secondary user.
    :type proxy_user_object: class[khoros.objects.users.ImpersonatedUser], None
    :returns: The API response in JSON format
    :raises: :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.APIConnectionError`,
             :py:exc:`khoros.errors.exceptions.POSTRequestError`,
             :py:exc:`khoros.errors.exceptions.PayloadMismatchError`
    """
    return add_subscription(khoros_object, msg_id, 'message', proxy_user_object=proxy_user_object)


def subscribe_to_product(khoros_object, product_id, proxy_user_object=None):
    """This function subscribes the current user to a product.

    .. versionchanged:: 4.0.0
       Introduced the ``proxy_user_object`` parameter to allow API requests to be performed on behalf of other users.

    .. versionadded:: 3.5.0

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :param product_id: The unique identifier for a product
    :type product_id: str, int
    :param proxy_user_object: Instantiated :py:class:`khoros.objects.users.ImpersonatedUser` object to perform the
                              API request on behalf of a secondary user.
    :type proxy_user_object: class[khoros.objects.users.ImpersonatedUser], None
    :returns: The API response in JSON format
    :raises: :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.APIConnectionError`,
             :py:exc:`khoros.errors.exceptions.POSTRequestError`,
             :py:exc:`khoros.errors.exceptions.PayloadMismatchError`
    """
    return add_subscription(khoros_object, product_id, 'product', proxy_user_object=proxy_user_object)



# -*- coding: utf-8 -*-
"""
:Module:            khoros.structures.categories
:Synopsis:          This module contains functions specific to categories within the Khoros Community platform
:Usage:             ``from khoros.structures import categories``
:Example:           ``category_id = categories.get_category_id(url)``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     17 Jul 2020
"""

import warnings

from . import base
from ..utils import log_utils
from .. import api, liql, errors

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)


def create(khoros_object, category_id, category_title, parent_id=None, return_json=True):
    """This function creates a new category.

    .. versionadded:: 2.5.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param category_id: The Category ID of the new category (e.g. ``video-games``)
    :type category_id: str
    :param category_title: The title of the new category (e.g. ``Video Games``)
    :type category_title: str
    :param parent_id: The Category ID of the parent category (optional)
    :type parent_id: str, None
    :param return_json: Determines whether or not the response should be returned in JSON format (``True`` by default)
    :type return_json: bool
    :returns: The response from the API call
    :raises: :py:exc:`ValueError`, :py:exc:`khoros.errors.exceptions.POSTRequestError`,
             :py:exc:`khoros.errors.exceptions.APIConnectionError`
    """
    parent_url = f"categories/id/{parent_id}/" if parent_id else ""
    endpoint = f"{parent_url}categories/add"
    query_params = {
        'category.id': category_id,
        'category.title': category_title
    }
    return api.make_v1_request(khoros_object, endpoint, query_params, 'POST', return_json)


def get_category_id(url):
    """This function retrieves the Category ID for a given category when provided its URL.

    .. versionchanged:: 2.6.0
       The function was refactored to leverage the :py:func:`khoros.structures.base.get_structure_id` function.

    :param url: The URL from which to parse out the Category ID
    :type url: str
    :returns: The Category ID retrieved from the URL
    :raises: :py:exc:`khoros.errors.exceptions.InvalidURLError`
    """
    return base.get_structure_id(url)


def get_total_count(khoros_object):
    """This function returns the total number of categories within the Khoros Community environment.

    .. versionadded:: 2.6.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :returns: The total number of categories as an integer
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`
    """
    return liql.get_total_count(khoros_object, 'categories')


def get_total_category_count(khoros_object):
    """This function returns the total number of categories within the Khoros Community environment.

    .. deprecated:: 2.6.0
       Use the :py:func:`khoros.structures.categories.get_total_count` function instead.

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :returns: The total number of categories as an integer
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`
    """
    warnings.warn("The 'khoros.structures.categories.get_total_category_count' function has been deprecated by the"
                  "'khoros.structures.categories.get_total_count' function and will be removed in a future release.",
                  DeprecationWarning)
    return get_total_count(khoros_object)


def category_exists(khoros_object, category_id=None, category_url=None):
    """This function checks to see if a category exists.

    .. versionadded:: 2.7.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param category_id: The ID of the category to check
    :type category_id: str, None
    :param category_url: The URL of the category to check
    :type category_url: str, None
    :returns: Boolean value indicating whether or not the category already exists
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return base.structure_exists(khoros_object, 'category', category_id, category_url)


def get_category_details(khoros_object, identifier, first_item=True):
    """This function returns a dictionary of category configuration settings.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str
    :param first_item: Filters the response data to the first item returned (``True`` by default)
    :type first_item: bool
    :returns: The category details within a dictionary
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return base.get_details(khoros_object, identifier, 'category', first_item)


def get_category_field(khoros_object, field, identifier=None, category_details=None):
    """This function returns a specific category field from the Khoros Community API.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param field: The field from the :py:class:`khoros.structures.base.Mapping` class whose value should be returned
    :type field: str
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: The data captured from the :py:func:`khoros.structures.base.get_details` function
    :type category_details: dict, None
    :returns: The requested field in its native format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return base.get_structure_field(khoros_object, field, identifier, structure_type='category',
                                    details=category_details)


def get_url(khoros_object, category_id=None, category_details=None):
    """This function retrieves the URL of a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param category_id: The ID of the category to be evaluated (optional if ``category_details`` dictionary is provided)
    :type category_id: str, None
    :param category_details: The data captured from the :py:func:`khoros.structures.base.get_details` function
    :type category_details: dict, None
    :returns: The full URL of the category
    :raises: :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    category_url = get_category_field(khoros_object, 'view_href', category_id, category_details)
    if '://' not in category_url:
        base_url = khoros_object.core['community_url']
        category_url = f"{base_url}{get_category_field(khoros_object, 'view_href', category_id, category_details)}"
    return category_url


def get_title(khoros_object, identifier=None, full_title=True, short_title=False, category_details=None):
    """This function retrieves the full and/or short title of the category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param full_title: Return the full title of the category (``True`` by default)
    :type full_title: bool
    :param short_title: Return the short title of the category (``False`` by default)
    :type short_title: bool
    :param category_details: Dictionary containing category details (optional)
    :type category_details: dict, None
    :returns: The category title(s) as a string or a tuple of strings
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    if not full_title and not short_title:
        exc_msg = "Must return at least the full title or the short title."
        raise errors.exceptions.MissingRequiredDataError(exc_msg)
    if not category_details:
        category_details = get_category_details(khoros_object, identifier)
    titles = (category_details['title'], category_details['short_title'])
    if not short_title:
        titles = titles[0]
    elif not full_title:
        titles = titles[1]
    return titles


def get_description(khoros_object, identifier=None, category_details=None):
    """This function retrieves the description for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The description in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'description', identifier, category_details)


def get_parent_type(khoros_object, identifier=None, category_details=None):
    """This function retrieves the parent type for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The parent type in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'parent_type', identifier, category_details)


def get_parent_id(khoros_object, identifier=None, category_details=None):
    """This function retrieves the parent ID for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The parent ID in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'parent_id', identifier, category_details)


def get_parent_url(khoros_object, identifier=None, category_details=None):
    """This function retrieves the parent URL for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The parent URL in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'parent_view_href', identifier, category_details)


def get_root_type(khoros_object, identifier=None, category_details=None):
    """This function retrieves the root category type for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The root category type in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'root_type', identifier, category_details)


def get_root_id(khoros_object, identifier=None, category_details=None):
    """This function retrieves the root category ID for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The root category ID in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'root_id', identifier, category_details)


def get_root_url(khoros_object, identifier=None, category_details=None):
    """This function retrieves the root category URL for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The root category URL in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'root_view_href', identifier, category_details)


def get_language(khoros_object, identifier=None, category_details=None):
    """This function retrieves the defined language for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The language (e.g. ``en``) in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'language', identifier, category_details)


def is_hidden(khoros_object, identifier=None, category_details=None):
    """This function identifies whether or not a given category is hidden.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: Boolean value indicating if the category is hidden
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'hidden', identifier, category_details)


def get_views(khoros_object, identifier=None, category_details=None):
    """This function retrieves the total view count for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The total number of views
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'views', identifier, category_details)


def friendly_date_enabled(khoros_object, identifier=None, category_details=None):
    """This function identifies if friendly dates are enabled for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: Boolean indicating if friendly dates are enabled
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'friendly_date_enabled', identifier, category_details)


def get_friendly_date_max_age(khoros_object, identifier=None, category_details=None):
    """This function retrieves the maximum age where friendly dates should be used (if enabled) for a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: Integer representing the number of days the friendly date feature should be leveraged if enabled
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'friendly_date_max_age', identifier, category_details)


def get_active_skin(khoros_object, identifier=None, category_details=None):
    """This function retrieves the skin being used with a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The name of the active skin in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'skin', identifier, category_details)


def get_depth(khoros_object, identifier=None, category_details=None):
    """This function retrieves the depth of a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The depth of the category as an integer
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'depth', identifier, category_details)


def get_position(khoros_object, identifier=None, category_details=None):
    """This function retrieves the position of a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The position of the category as an integer
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    return get_category_field(khoros_object, 'position', identifier, category_details)


def get_creation_date(khoros_object, identifier=None, category_details=None):
    """This function retrieves the creation date of a given category.

    .. versionadded:: 2.1.0

    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros]
    :param identifier: The Category ID or Category URL with which to identify the category
    :type identifier: str, None
    :param category_details: Dictionary containing community details (optional)
    :type category_details: dict, None
    :returns: The creation of the category in string format
    :raises: :py:exc:`khoros.errors.exceptions.GETRequestError`,
             :py:exc:`khoros.errors.exceptions.InvalidFieldError`,
             :py:exc:`khoros.errors.exceptions.InvalidStructureTypeError`,
             :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`
    """
    # TODO: Allow a format to be specified and the ability to parse as a datetime object if needed
    return get_category_field(khoros_object, 'creation_date', identifier, category_details)

